#####################################################
## A function to simulate data with binary outcome ##
#####################################################

# n: sample size
# p1: number of predictors in modality 1
# p2: number of predictors in modality 2
# prel1: number of relevant predictors in modality 1
# prel2: number of relevant predictors in modality 2
# beta1, beta2: effect strength
# prop1: probability Y=1

simulclassif<-function(n=NULL,p1=NULL,p2=NULL,prel1,prel2,beta1,beta2,prop1=0.5,X1=NULL,X2=NULL,sigma=NULL)
{
linpredX1<-0
linpredX2<-0

if ((is.null(X1)+is.null(X2))==1)
 {
 stop("X1 and X2 must be either both specified or both NULL")
 }


if (is.null(X1)&is.null(X2))
 {
 if (is.null(sigma))
  {
  sigma<-diag(rep(1,p1+p2))
  }
 Y<-rbinom(n,1,prop1)
 X<-rmvnorm(n,mean=rep(0,p1+p2),sigma=sigma)
 X1<-X[,1:p1]
 X2<-X[,(p1+1):(p1+p2)]
 }

if (!is.null(X1)&!is.null(X2))
 {
 if (is.null(n))
  {
  Y<-rbinom(nrow(X1),1,prop1)
  }
 
 if (!is.null(n))
  {
  if (n%%nrow(X1)!=0)
   stop("n has to be divisible by nrow(X1)")
  else
   {
   Y<-rbinom(nrow(X1),1,prop1)
   X1<-matrix(X1,n,ncol(X1),byrow=TRUE)
   X2<-matrix(X2,n,ncol(X2),byrow=TRUE)
   }
  }

 }


if (prel1>0&beta1!=0)
  {
  ind.rel<-sample(ncol(X1),prel1)
  X1<-cbind(X1[,ind.rel],X1[,-ind.rel])
  X1[Y==1,1:prel1]<-X1[Y==1,1:prel1]+beta1
  }

if (prel2>0&beta2!=0)
 { 
 ind.rel<-sample(ncol(X2),prel2)
 X2<-cbind(X2[,ind.rel],X2[,-ind.rel])
 X2[Y==1,1:prel2]<-X2[Y==1,1:prel2]+beta2
 }
return(cbind(Y,X1,X2))
}




##############################
# Generate covariance matrix #
##############################

covMatrix<-function (p, b, independentModalities = FALSE, rho = 0.4) 
{
    stopifnot(all(p%%b == 0))
    nrModalities <- length(p)
    if (independentModalities) 
        x <- diag(nrModalities * b)
    else x <- kronecker(matrix(1, nrModalities, nrModalities), 
        diag(b))
    y <- matrix(ncol = 0, nrow = 0)
    for (i in 1:nrModalities) for (j in 1:b) y <- matrixcalc::direct.sum(y, 
        matrix(1, 1, p[i]/b))
    rho * (t(y) %*% x %*% y) + (1 - rho) * diag(sum(p))
}



##################################
# A function to run a simulation #
##################################


# n: sample size
# p1: number of predictors in modality 1
# p2: number of predictors in modality 2
# prel1: number of relevant predictors in modality 1
# prel2: number of relevant predictors in modality 2
# beta1, beta2: effect strength
# prop1: probability Y=1

# family: should be "gaussian" for continuous Y or "binomial" for binary Y
# nfolds: number of folds of CV procedure
# ncv: number of repetitions of CV
# type.measure: should be "class" or "auc" if Y is binary, or "mse" is Y is continuous

# niter: number of simulated datasets
# pflist: a list of penalty factors vectors of the format pflist=list(c(1,1),c(1,10),c(10,1)). 
# nzeromax: the maximal number of predictors allowed in the final model. Default is +Inf, i.e. the best model is selected based on CV error without restriction.
# plot: plot=TRUE outputs plots of CV errors and number of included predictors for each modality


simulation<-function(n=NULL,p1=NULL,p2=NULL,beta1,beta2,prel1,prel2,prop1=NULL,
			family,nfolds,ncv,type.measure,
			niter, pflist,nzeromax=Inf,sigma=NULL,
			plot=FALSE,X1=NULL,X2=NULL,
			competitors="notall",alpha=1,dir)
{
if (is.null(X1)&is.null(X2))
 {
 namesimu<-paste("n",n,"p",p1,"p",p2,"prel",prel1,"prel",prel2,"beta1",beta1,"beta2",beta2,"prop1",prop1,"nzeromax",nzeromax,type.measure,sep="")
 }
else
 {
 namesimu<-paste("Novartis_","prel",prel1,"prel",prel2,"beta1",beta1,"beta2",beta2,"prop1",prop1,"nzeromax",nzeromax,type.measure,sep="")
 p1<-ncol(X1)
 p2<-ncol(X2)
 n<-nrow(X1)
 }

blocks<-list(block1=1:p1,block2=(p1+1):(p1+p2))

nw<-length(pflist)
bestpf<-numeric(niter)

nzero<-matrix(NA,niter,nw+1)
nzero1<-matrix(NA,niter,nw+1)
nzerotrue1<-matrix(NA,niter,nw+1)
nzero2<-matrix(NA,niter,nw+1)
nzerotrue2<-matrix(NA,niter,nw+1)
cvm<-matrix(NA,niter,nw+1)
testm<-matrix(NA,niter,nw+1)
aucm<-matrix(NA,niter,nw+1)


pfnames<-c()
for (j in 1:nw)
 {
 pfnames<-c(pfnames,paste(pflist[[j]],collapse="-"))
 }
colnames(nzero)<-c(pfnames,"CV")
colnames(nzero1)<-c(pfnames,"CV")
colnames(nzero2)<-c(pfnames,"CV")
colnames(cvm)<-c(pfnames,"CV")
colnames(testm)<-c(pfnames,"CV")
colnames(aucm)<-c(pfnames,"CV")


testmcompetitors<-matrix(NA,niter,2)
aucmcompetitors<-matrix(NA,niter,2)
nzerocompetitors<-matrix(NA,niter,2)
colnames(aucmcompetitors)<-c("sgl","s")
colnames(testmcompetitors)<-c("sgl","s")
colnames(nzerocompetitors)<-c("sgl","s")

if (plot)
 {
 pdf(paste(dir,"/",namesimu,".pdf",sep=""))
 }
for (i in 1:niter)
 { 
 print(i)
 set.seed(i)


 #####################
 # simulate the data #
 #####################
 if (!is.null(prop1))
  {
  data.all<-simulclassif(n=n+5000,p1=p1,p2=p2,beta1=beta1,beta2=beta2,prel1=prel1,prel2=prel2,prop1=prop1,X1=X1,X2=X2,sigma=sigma)
  data.train<-data.all[1:n,]
  data.test<-data.all[-(1:n),]
  }
 else
  {
  data.all<-simulregr(n=n+5000,p1=p1,p2=p2,beta1=beta1,beta2=beta2,prel1=prel1,prel2=prel2)
  data.train<-data.all[1:n,]
  data.test<-data.all[-(1:n),]
  } 

 
 ####################################
 # competitor 1: sparse group lasso # 
 ####################################
 
 index<-c(rep(1,p1),rep(2,p2))
 if (family=="binomial")
 {
   type<-"logit"
 }
 resultsgl<-cvSGL(data=list(x=as.matrix(data.train)[,-1],y=data.train[,1]),
                  index=index,type=type,nfold=nfolds,thres=0.01)
 resl<-SGL(data=list(x=as.matrix(data.train)[,-1],y=data.train[,1]),index=index,type=type,
           lambdas=resultsgl$lambdas,thres=0.01)
 predtestsgl<-predictSGL(resl,as.matrix(data.test)[,-1],lam=which.min(resultsgl$lldiff)[1])
 nzerocompetitors[i,1]<-sum(resl$beta[,which.min(resultsgl$lldiff)[1]]!=0)
 testmcompetitors[i,1]<-sum(data.test[,1]!=as.numeric(predtestsgl>0.5))/nrow(data.test)
 aucmcompetitors[i,1]<-my.auc(predtestsgl,data.test[,1])
 

 #################################
 # competitor 2: separate models # 
 #################################

 model1<-cvr.glmnet(Y=data.train[,1],X=data.train[,blocks[[1]]+1],family=family,standardize=TRUE,ncv=ncv,nfolds=nfolds,type.measure=type.measure)
 model2<-cvr.glmnet(Y=data.train[,1],X=data.train[,blocks[[2]]+1],family=family,standardize=TRUE,ncv=ncv,nfolds=nfolds,type.measure=type.measure)
 if (type.measure!="auc")
  {
  ind.bestlambda1<-which.min(model1$cvm)[1]
  ind.bestlambda2<-which.min(model2$cvm)[1]
  }
 if (type.measure=="auc")
  {
  ind.bestlambda1<-which.max(model1$cvm)[1]
  ind.bestlambda2<-which.max(model2$cvm)[1]
  }

 coeff1<-model1$coeff[-1,ind.bestlambda1]  
 coeff2<-model2$coeff[-1,ind.bestlambda2]
  
 nzerocompetitors[i,2]<-sum(coeff1!=0)+sum(coeff2!=0)

 if (nzerocompetitors[i,2]!=0)
 {
 if (all(coeff1==0))
  {
  data.train2<-data.frame(y=data.train[,1],x2=data.train[,blocks[[2]]+1]%*%coeff2)
  data.test2<-data.frame(y=data.test[,1],x2=data.test[,blocks[[2]]+1]%*%coeff2)
  }

 if (all(coeff2==0))
  {
  data.train2<-data.frame(y=data.train[,1],x1=data.train[,blocks[[1]]+1]%*%coeff1)
  data.test2<-data.frame(y=data.test[,1],x1=data.test[,blocks[[1]]+1]%*%coeff1)
  }

 if (!all(coeff1==0)&!all(coeff2==0))
  {
  data.train2<-data.frame(y=data.train[,1],x1=data.train[,blocks[[1]]+1]%*%coeff1,x2=data.train[,blocks[[2]]+1]%*%coeff2)
  data.test2<-data.frame(y=data.test[,1],x1=data.test[,blocks[[1]]+1]%*%coeff1,x2=data.test[,blocks[[2]]+1]%*%coeff2)
  }

 # GLM
 predtest<-predict(object=glm(data=data.train2,formula=y~.,family="binomial"),data.test2)
 testmcompetitors[i,2]<-sum(data.test[,1]!=as.numeric(predtest>0))/nrow(data.test)
 aucmcompetitors[i,2]<-my.auc(predtest,data.test[,1]) 
 }


 ##################
 # our new method #
 ##################

 result<-cvr2.ipflasso(Y=data.train[,1],X=data.train[,-1],family=family,standardize=TRUE,
			blocks=blocks,pflist=pflist,nzeromax=nzeromax,
			nfolds=nfolds,ncv=ncv,type.measure=type.measure,
			plot=plot,alpha=alpha)
 for (j in 1:nw)
  {
  aj<-result$a[[j]]
  beta<-aj$coeff[-1,which.min(aj$cvm)]
  nzero[i,j]<-sum(beta!=0)
  nzero1[i,j]<-sum(beta[blocks[[1]]]!=0)
  if (prel1>0)
   {
   nzerotrue1[i,j]<-sum(beta[blocks[[1]]][1:prel1]!=0)
   }
  nzero2[i,j]<-sum(beta[blocks[[2]]]!=0)
  if (prel2>0)
   {
   nzerotrue2[i,j]<-sum(beta[blocks[[2]]][1:prel2]!=0)
   }
  cvm[i,j]<-min(aj$cvm)

  predtest<-ipflasso.predict(aj,Xtest=data.test[,-1])
  
  if (family=="gaussian")
   {
   testm[i,j]<-sum((data.test[,1]-predtest$linpredtest)^2)/5000
   }
  if (family=="binomial")
   {
   testm[i,j]<-sum(data.test[,1]!=predtest$classtest)/5000
   aucm[i,j]<-my.auc(as.numeric(predtest$linpredtest),data.test[,1])
   }
  }
 
 nzero[i,nw+1]<-nzero[i,result$ind.bestpf]
 nzero1[i,nw+1]<-nzero1[i,result$ind.bestpf]
 nzerotrue1[i,nw+1]<-nzerotrue1[i,result$ind.bestpf]
 nzero2[i,nw+1]<-nzero2[i,result$ind.bestpf]
 nzerotrue2[i,nw+1]<-nzerotrue2[i,result$ind.bestpf]
 cvm[i,nw+1]<-cvm[i,result$ind.bestpf]
 testm[i,nw+1]<-testm[i,result$ind.bestpf]
 aucm[i,nw+1]<-aucm[i,result$ind.bestpf]
 bestpf[i]<-pfnames[result$ind.bestpf]
 
 }

if (plot)
 {
 dev.off()
 }

result<-list(nzero=nzero,nzero1=nzero1,nzerotrue1=nzerotrue1,nzero2=nzero2,nzerotrue2=nzerotrue2,nzerocompetitors=nzerocompetitors,cvm=cvm,testm=testm,aucm=aucm,testmcompetitors=testmcompetitors,aucmcompetitors=aucmcompetitors,bestpf=bestpf,
		p1=p1,p2=p2,prel1=prel1,prel2=prel2,beta1=beta1,beta2=beta2)
save(result,file=paste(dir,"/",namesimu,".RData",sep=""))
return(result)
}

